#!/usr/bin/perl -w

# History
# =======
# V0.8:
# - Regelung "alle MagTI-LVAs sind BakkTI-LVAs" gestrichen
# - Grep nach "Soft Skill & Gender Studies"-LVAs gendert
# - Regelung "alle BakkSE-LVAs sind MagSI-LVAs" hinzugefgt
# V0.7:
# - "MIE" an zwei Stellen ausgebessert auf "SIE" (Grund: Typo)
# V0.6:
# - Start of History

$version="0.8";
$DAT="dat";
$TITEL="$DAT/titel.dat";
$FACH="$DAT/pruefach.dat";
$TYP="$DAT/typ.dat";
$LVA="$DAT/lva.dat";
$WHO="$DAT/who.dat";
$TEX="lvadat.tex";
$STUDIUM="$DAT/studium.dat";
$KAT="$DAT/kategorie.dat";

sub uml {
   $_ = shift;
   s//A{/;s//O{/;s//U{/;
   s//a{/;s//o{/;s//u{/;
   s//s{/;
   return $_;
};

sub lva_cmp {
   ($tita, $typa, $swsa) = split /-/,$a;
   ($titb, $typb, $swsb) = split /-/,$b;
   $Stitel{$tita} cmp $Stitel{$titb}
   or $Styp{$typa} cmp $Styp{$typb}
   or $swsa <=> $swsb;
};

sub kat_cmp {
   $Kshort{$a} cmp $Kshort{$b};
};

sub in_cmp {
   ($kata) = ($a =~ /(..).*/);
   ($katb) = ($b =~ /(..).*/);
   $Kshort{$kata} cmp $Kshort{$katb};
};

sub Xs2tex {
   $X=shift;
   $t = '';
   foreach (@_) {
      $t .= "\\${X}{$_}";
   };
   return $t;
}

sub Sws {
   $sum=0;
   foreach (@_) {
      ($tit, $typ, $sws) = split /-/;
      $sum += $sws;
   };
   return $sum;
}

sub mtime {
   return localtime((stat(@_))[9]);
}

open TEX, "> $TEX" or die "Can't open file $TEX";

$d=localtime;
$LVAver=mtime($LVA);
$TITELver=mtime($TITEL);
$TYPver=mtime($TYP);
$FACHver=mtime($FACH);
$STUDIUMver=mtime($STUDIUM);
$KATver=mtime($KAT);
$WHOver=mtime($WHO);
print TEX <<"EOF";
% DO NOT EDIT THIS FILE.
% Created by $0, version $version,
% on $d from the files
%    $LVA	($LVAver),
%    $TITEL	($TITELver),
%    $TYP	($TYPver),
%    $FACH	($FACHver),
%    $STUDIUM	($STUDIUMver),
%    $WHO	($WHOver),
%    $KAT	($KATver).

EOF

open TITEL or die "Can't open file $TITEL";
TITEL: while ( <TITEL> ) {
   next TITEL if /^#/;
   chomp;
   ( $key, $titel ) = split /\t/;
   $Stitel{$key} = lc(uml($titel));
   printf TEX "\\titel{%s}{%s}\n",$key,$titel;
};
close TITEL;

open TYP or die "Can't open file $TYP";
@Typs=();
TYP: while ( <TYP> ) {
   next TYP if /^#/;
   chomp;
   ( $key, $typsrt, $typlong ) = split /\t/;
   $Styp{$key} = $typsrt;
   @Typs=(@Typs, $key);
   printf TEX "\\typ{%s}{%s}\n",$key,$typlong;
};
close TYP;
@Typs=sort @Typs;
printf TEX "\\typsalle{%d}{%s}\n", scalar(@Typs), Xs2tex('Y',@Typs);

open WHO or die "Can't open file $WHO";
@Whos=();
WHO: while ( <WHO> ) {
   next WHO if /^#/;
   chomp;
   ( $who, $uni ) = split /\t/;
   $Uni{$who} = $uni;
   @Whos=(@Whos, $who);
   printf TEX "\\who{%s}{%s}\n",$who,$uni;
};
close WHO;
@Whos=sort @Whos;
printf TEX "\\whosalle{%d}{%s}\n", scalar(@Whos), Xs2tex('W',@Whos);

open FACH or die "Can't open file $FACH";
@Faecher=();
FACH: while ( <FACH> ) {
   next FACH if /^#/;
   chomp;
   ( $key, $titel ) = split /\t/;
   @Faecher=(@Faecher, $key);
   printf TEX "\\fach{%s}{%s}\n",$key,$titel;
};
close FACH;

open STUDIUM or die "Can't open file $STUDIUM";
@Studs=();
STUDIUM: while ( <STUDIUM> ) {
   next STUDIUM if /^#/;
   chomp;
   ( $key, $short, $long ) = split /\t/;
   @Studs=(@Studs, $key);
   printf TEX "\\stud{%s}{%s}{%s}\n",$key,$short,$long;
};
close STUDIUM;
@Studs=sort @Studs;
printf TEX "\\studsalle{%d}{%s}\n", scalar(@Studs), Xs2tex('S',@Studs);

open KAT or die "Can't open file $KAT";
@Kats=();
KAT: while ( <KAT> ) {
   next KAT if /^#/;
   chomp;
   ( $key, $short, $long ) = split /\t/;
   @Kats=(@Kats, $key);
   $Kshort{$key}=$short;
   printf TEX "\\kat{%s}{%s}{%s}\n",$key,$short,$long;
};
close KAT;
@Kats=sort kat_cmp @Kats;
printf TEX "\\katsalle{%d}{%s}\n", scalar(@Kats), Xs2tex('K',@Kats);

open LVA;
@Lvas=();
%Lin=();
%Lby=();
LVA: while ( <LVA> ) {
   next LVA if /^#/;
   chomp;
   ($tit, $typ, $sws, $in, $by) = split /\t/;
   $key=$tit."-".$typ."-".$sws;
   @Lvas = (@Lvas, $key);
   $Lin{$key} = ",".$in;
   $Lby{$key} = ",".$by;
   $LIN{$tit."-".$typ} .= ",".$in;
};
close LVA;

@Studsems=();
foreach $l (@Lvas) {
   ($tit, $typ, $sws) = split /-/,$l;
   $Linx = $Lin{$l};
   #
   # Jede Pflicht- oder Wahllva eines Bakkstudiums (=D|M|S|T|Z)
   # ist Wahllva im Bakk "Data Eng.&Stat" (=DW), falls nicht bereits
   # eine Lva gleichen Titels und gleichen Typs in diesem Studium (=D) vorkommt.
   #
   $Linx .= ',DW?DWa' if ($Lin{$l} =~ /,(D|M|S|T|Z)/)
            and ($LIN{$tit."-".$typ} !~ /,D/);
   #
   # Jede Pflicht- oder Wahllva eines Bakkstudiums (=D|M|S|T|Z)
   # ist Wahllva im Bakk "Medizinische Inf." (=ZW), falls nicht bereits
   # eine Lva gleichen Titels und gleichen Typs in diesem Studium (=Z) vorkommt.
   #
   $Linx .= ',ZW?ZWa' if ($Lin{$l} =~ /,(D|M|S|T|Z)/)
            and ($LIN{$tit."-".$typ} !~ /,Z/);
   #
   # Jede Pflichtlva (=P) eines Bakkstudiums (=D|M|S|T|Z) sowie die Lvas der Kataloge MCGI, MDes, SIE
   # sind Wahllva im Bakk "SW&Inf Eng", Zweig "SW Eng" (=SX), falls nicht bereits
   # eine LVA gleichen Titels und gleichen Typs in diesem Zweig (=SP|SQ|SX) vorkommt.
   # Bis Version 0.6 stand hier aufgrund eines Typos "MIE" statt "SIE"
   #
   $Linx .= ',SX?SXa' if ($Lin{$l} =~ /,(D|M|S|T|Z)P|,...(MCGI|MDes|SIE)/)
            and ($LIN{$tit."-".$typ} !~ /,S(P|Q|X)/);
#
#  Die folgende Regelung wurde ab Studienplanversion 2.3 (Version 0.8
#  dieses Skripts) gestrichen.
#   #
#   # Jede Pflicht- oder Wahllva des Magstudiums "Technische Informatik" (=U)
#   # ist Wahllva im Bakk "Technische Informatik" (=TW).
#   #
#   $Linx .= ',TW?TWa' if ($Lin{$l} =~ /,U/);
   #
   # Jede Pflicht- oder Wahllva des Bakkstudiums "Software & Information Engineering" (=S)
   # ist Wahllva im Mag "Software Engineering & Internet Computing" (=RW).
   #
   $Linx .= ',RW?RWa' if ($Lin{$l} =~ /,S/);
   #
   # Jede Pflicht- oder Wahllva des Bakkstudiums "Technische Informatik" (=T)
   # ist Wahllva im Mag "Technische Informatik" (=UW).
   #
   $Linx .= ',UW?UWa' if ($Lin{$l} =~ /,T/);
   #
   # Die Lvas des Katalogs "Softs Skills & Gender Studies" (=SSGS) sind Wahllvas
   # in allen Bakkalaureatsstudien (=DW,MX,MY,SX,SY,TW,ZW) und im Magstudium "Medizinische Inf."
   # (=YW)
   #
   $Linx .= ',DW?DWa,MX?MXa,MY?MYa,SX?SXa,SY?SYa,TW?TWa,ZW?ZWa,YW?YWa'
            if ($Lin{$l} =~ /,...SG/);
   #
   printf TEX "\\lva{%s}{", $l;
   @ins = split /,/, $Linx;
   shift @ins;
   @instmp = sort in_cmp @ins;
   $prev = 'undefiniert';
   @ins = grep ($_ ne $prev && ($prev=$_), @instmp);
   foreach $s (@ins) {
      ($studsem, $fach) = ($s =~ /(...)(.+)/);
      @Faecher = (@Faecher, $fach);
      @Studsems = (@Studsems, $studsem);
      print TEX "\\I{$s}";
   }
   printf TEX "}{";
   @bys = split /,/, $Lby{$l};
   shift @bys;
   $unis = "";
   foreach $b (sort @bys) {
      print TEX "\\B{$b}";
      $unis = $unis . "," . $Uni{$b};
   };
   printf TEX "}{";
   @u = split /,/, $unis;
   shift @u;
   @v = sort @u;
   $prev = 'undefiniert';
   @u = grep ($_ ne $prev && ($prev=$_), @v);
   foreach $w (@u) {
      print TEX "\\U{$w}";
   };
   printf TEX "}\n";
};

@Lvas = sort lva_cmp @Lvas;
printf TEX "\\lvasalle{%d}{%s}\n", scalar(@Lvas), Xs2tex('L',@Lvas);

@f = sort @Faecher;
$prev = 'undefiniert';
@Faecher = grep ($_ ne $prev && ($prev=$_), @f);

printf TEX "\\fachsalle{%d}{%s}\n", scalar(@Faecher), Xs2tex('F',@Faecher);

foreach $f (@Faecher) {
   @lvas = grep (($Lin{$_} =~ /,...\Q$f\E(,|$)/), @Lvas);
   printf TEX "\\lvasbyfach{%s}{%.1f}{%s}\n", $f, Sws(@lvas), Xs2tex('L',@lvas);
};

@s = sort @Studsems;
$prev = 'undefiniert';
@Studsems = grep ($_ ne $prev && ($prev=$_), @s);
foreach $s (@Studsems) {
   @lvas = grep (($Lin{$_} =~ /,\Q$s\E/), @Lvas);
   printf TEX "\\lvasbysem{%s}{%.1f}{%s}\n", $s, Sws(@lvas), Xs2tex('L',@lvas);
};

close TEX;
